@@ -196,6 +196,14 @@ drawer_click (GtkWidget *w, Drawer *drawer)
 		panel_toplevel_unhide (drawer->toplevel);
 }
 
+static void
+toplevel_destroyed (GtkWidget *widget,
+		    Drawer    *drawer)
+{
+	drawer->toplevel = NULL;
+	gtk_widget_destroy (drawer->button);
+}
+
 static void
 destroy_drawer (GtkWidget *widget,
 		Drawer    *drawer)
@@ -206,7 +214,7 @@ destroy_drawer (GtkWidget *widget,
 	drawer->properties = NULL;
 
 	if (drawer->toplevel)
-		panel_toplevel_detach (drawer->toplevel);
+		gtk_widget_destroy (GTK_WIDGET (drawer->toplevel));
 	drawer->toplevel = NULL;
 
 	if (drawer->close_timeout_id)
@@ -518,6 +526,8 @@ create_drawer_applet (PanelToplevel    *toplevel,
 			  G_CALLBACK (destroy_drawer), drawer);
 	g_signal_connect (drawer->button, "key_press_event",
 			  G_CALLBACK (key_press_drawer), drawer);
+	g_signal_connect (toplevel, "destroy",
+			  G_CALLBACK (toplevel_destroyed), drawer);
 
 	gtk_widget_show (drawer->button);
 
@@ -636,6 +646,7 @@ load_drawer_applet (char          *toplevel_id,
 
 	panel_widget_add_forbidden (panel_toplevel_get_panel_widget (drawer->toplevel));
 	panel_widget_set_applet_expandable (panel_widget, GTK_WIDGET (drawer->button), FALSE, TRUE);
+	panel_widget_set_applet_size_constrained (panel_widget, GTK_WIDGET (drawer->button), TRUE);
 
 	gtk_tooltips_set_tip (panel_tooltips, drawer->button, drawer->tooltip, NULL);
 	gtk_window_present (GTK_WINDOW (drawer->toplevel));
@@ -684,12 +695,6 @@ panel_drawer_create (PanelToplevel *toplevel,
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
 }
 
-void
-panel_drawer_delete (Drawer *drawer)
-{
-	panel_profile_delete_toplevel (drawer->toplevel);
-}
-
 void
 drawer_load_from_gconf (PanelWidget *panel_widget,
 			gboolean     locked,
