@@ -5,11 +5,6 @@
 
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-directory.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomeui/gnome-ditem-edit.h>
 
 #include "menu-ditem.h"
@@ -116,19 +111,11 @@ ditem_properties_close (GtkWidget *dialog,
 
 	saving_error = g_object_get_data (G_OBJECT (dedit), "SavingError");
 
-	if (saving_error) {
-		char *msg;
-		msg = g_strdup_printf ("<b>%s</b>\n\n%s: %s",
-			_("Cannot save changes to launcher"),
-			_("Details"), saving_error);
-
-		panel_error_dialog (
-			gtk_window_get_screen (GTK_WINDOW (dialog)),
-			"cannot_save_entry",
-			msg);
-
-		g_free (msg);
-	}
+	if (saving_error)
+		panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (dialog)),
+				    "cannot_save_entry",
+				    _("Cannot save changes to launcher"),
+				    saving_error);
 }
 
 static gboolean
@@ -197,7 +184,11 @@ panel_edit_dentry (const char *loc,
 	gtk_dialog_set_default_response (
 			GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	
 	dedit = gnome_ditem_edit_new ();
+	gtk_container_set_border_width (GTK_CONTAINER (dedit), 5);
 
 	gtk_widget_show (dedit);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
@@ -271,11 +262,15 @@ panel_edit_direntry (const char *dir,
 				GTK_RESPONSE_CLOSE,
 				NULL);
 
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 				"desktop_entry_properties", "Panel");
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 	
 	dedit = gnome_ditem_edit_new ();
+	gtk_container_set_border_width (GTK_CONTAINER (dedit), 5);
 	gtk_widget_show (dedit);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 			    dedit, TRUE, TRUE, 0);
@@ -448,10 +443,11 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 
 	/* check for valid name */
 	if (string_empty (gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME))) {
-		dialog = panel_error_dialog (
-				gtk_window_get_screen (GTK_WINDOW (d)),
-				"cannot_create_launcher",
-				_("You have to specify a name for the launcher."));
+		dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (d)),
+					     "cannot_create_launcher",
+					     _("Cannot create launcher"),
+					     _("You have to specify a name."));
+
 		g_signal_connect_swapped (G_OBJECT (dialog),
 					  "destroy",
 					  G_CALLBACK (panel_pop_window_busy),
@@ -464,10 +460,10 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 	     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC))) ||
 	    (gnome_desktop_item_get_entry_type (ditem) == GNOME_DESKTOP_ITEM_TYPE_LINK &&
 	     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_URL)))) {
-		dialog = panel_error_dialog (
-				gtk_window_get_screen (GTK_WINDOW (d)),
-				"cannot_create_launcher",
-				_("You have to specify a valid URL or command."));
+		dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (d)),
+					     "cannot_create_launcher",
+					     _("Cannot create launcher"),
+					     _("You have to specify a valid URL or command."));
 		g_signal_connect_swapped (G_OBJECT (dialog),
 					  "destroy",
 					  G_CALLBACK (panel_pop_window_busy),
@@ -495,17 +491,11 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 				 TRUE /* force */,
 				 &error);
 	if (error) {
-		char *msg;
-		msg = g_strdup_printf ("<b>%s</b>\n\n%s: %s",
-			_("Cannot save menu item to disk"),
-			_("Details"), error->message);
+		panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (d)),
+				    "cannot_save_menu_item" /* class */,
+				    _("Cannot save menu item to disk"),
+				    error->message);
 
-		panel_error_dialog (
-			gtk_window_get_screen (GTK_WINDOW (d)),
-			"cannot_save_menu_item" /* class */,
-			msg);
-
-		g_free (msg);
 		g_clear_error (&error);
 	}
 
@@ -525,11 +515,11 @@ panel_new_launcher (const char *item_loc,
 	GtkWidget *dee;
 
 	if (!is_item_writable (item_loc, NULL)) {
-		dialog = panel_error_dialog (
-				screen,
-				"cannot_create_launcher",
-				_("You can not create a new launcher at this location "
-				  "since the location is not writable."));
+		dialog = panel_error_dialog (screen,
+					     "cannot_create_launcher",
+					     _("Cannot create launcher"),
+					     _("You can not create a new launcher at this "
+					       "location since the location is not writable."));
 
 		return dialog;
 	}
@@ -541,11 +531,14 @@ panel_new_launcher (const char *item_loc,
 				GTK_STOCK_OK, GTK_RESPONSE_OK,
 				NULL);
 
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 			       "create_menu_item", "Panel");
 	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 	
 	dee = gnome_ditem_edit_new ();
+	gtk_container_set_border_width (GTK_CONTAINER (dee), 5);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), dee,
 			    TRUE, TRUE, GNOME_PAD_SMALL);
 
