@@ -232,7 +232,7 @@ stock_menu_item_new (const char *text,
 		item, panel_menu_icon_get_size (), stock_id, NULL, NULL, force_image);
 
         if (text) {
-                label = gtk_label_new (text);
+                label = gtk_label_new_with_mnemonic (text);
                 gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
                 gtk_container_add (GTK_CONTAINER (item), label);
         }
