@@ -1,22 +1,22 @@
 /*
- * panel-applet-frame.c:
+ * panel-applet-frame.c: panel side container for applets
  *
- * Copyright (C) 2001 Sun Microsystems, Inc.
+ * Copyright (C) 2001 - 2003 Sun Microsystems, Inc.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  *
  * Authors:
  *	Mark McLoughlin <mark@skynet.ie>
@@ -32,19 +32,25 @@
 #include <gdk/gdkx.h>
 
 #include "panel-applet-frame.h"
-#include "panel-gconf.h"
+#include "panel-profile.h"
 #include "panel-util.h"
 #include "panel.h"
-#include "session.h"
 #include "applet.h"
 #include "panel-marshal.h"
 #include "panel-background.h"
 
+#undef PANEL_APPLET_FRAME_DEBUG
+
 #define HANDLE_SIZE 10
 
 #define PANEL_STOCK_DONT_DELETE "panel-dont-delete"
 
-#undef PANEL_APPLET_FRAME_DEBUG
+#define PROPERTY_ORIENT     "panel-applet-orient"
+#define PROPERTY_SIZE       "panel-applet-size"
+#define PROPERTY_BACKGROUND "panel-applet-background"
+#define PROPERTY_FLAGS      "panel-applet-flags"
+#define PROPERTY_SIZE_HINTS "panel-applet-size-hints"
+
 
 struct _PanelAppletFramePrivate {
 	GNOME_Vertigo_PanelAppletShell  applet_shell;
@@ -53,13 +59,15 @@ struct _PanelAppletFramePrivate {
 
 	PanelWidget                    *panel;
 	AppletInfo                     *applet_info;
-	PanelOrient                     orient;
+	PanelOrientation                orientation;
 
 	gchar                          *iid;
 	gboolean			moving_focus_out;
 
 	GtkAllocation                   child_allocation;
 	GdkRectangle                    handle_rect;
+
+	guint                           has_handle : 1;
 };
 
 static GObjectClass *parent_class;
@@ -73,43 +81,115 @@ typedef enum {
 	APPLET_HAS_HANDLE   = 1 << 2,
 } PanelAppletFlags;
 
-void
-panel_applet_frame_save_to_gconf (PanelAppletFrame *frame,
-				  const char       *gconf_key)
+static void
+panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 {
-	GConfClient *client;
-	const char  *profile;
-	const char  *temp_key;
+	PanelWidget *panel_widget;
+	gboolean     locked;
+	gboolean     lockable;
+	gboolean     movable;
+	gboolean     removable;
 
-	client  = panel_gconf_get_client ();
-	profile = panel_gconf_get_profile ();
+	panel_widget = PANEL_WIDGET (GTK_WIDGET (frame)->parent);
 
-	temp_key = panel_gconf_full_key (PANEL_GCONF_APPLETS, profile, gconf_key, "bonobo_iid");
-	gconf_client_set_string (client, temp_key, frame->priv->iid, NULL);
+	lockable = panel_applet_lockable (frame->priv->applet_info);
+	movable = panel_applet_can_freely_move (frame->priv->applet_info);
+	removable = panel_profile_list_is_writable (PANEL_GCONF_APPLETS);
+
+	locked = panel_widget_get_applet_locked (panel_widget, GTK_WIDGET (frame));
+
+	bonobo_ui_component_set_prop (frame->priv->ui_component,
+				      "/popups/button3/placeholder/lock",
+				      "label",
+				      locked ? _("Un_lock") : _("_Lock"),
+				      NULL);
+
+	bonobo_ui_component_set_prop (frame->priv->ui_component,
+				      "/commands/LockAppletToPanel",
+				      "sensitive",
+				      lockable ? "1" : "0",
+				      NULL);
+
+	bonobo_ui_component_set_prop (frame->priv->ui_component,
+				      "/commands/RemoveAppletFromPanel",
+				      "sensitive",
+				      removable ? "1" : "0",
+				      NULL);
+
+	bonobo_ui_component_set_prop (frame->priv->ui_component,
+				      "/commands/MoveApplet",
+				      "sensitive",
+				      locked ? "0" : (movable ? "1" : "0"),
+				      NULL);
 }
 
-void
-panel_applet_frame_load_from_gconf (PanelWidget *panel_widget,
-				    gint         position,
-				    const char  *gconf_key)
+static void
+panel_applet_frame_set_flags_from_any (PanelAppletFrame *frame,
+				       const CORBA_any  *any)
 {
-	GConfClient *client;
-	const char  *profile;
-	const char  *temp_key;
-	char        *applet_iid;
+	gboolean major;
+	gboolean minor;
+	int      flags;
+	
+	flags = BONOBO_ARG_GET_SHORT (any);
 
-	g_return_if_fail (panel_widget != NULL);
-	g_return_if_fail (gconf_key != NULL);
+	major = flags & APPLET_EXPAND_MAJOR;
+	minor = flags & APPLET_EXPAND_MINOR;
 
-	client  = panel_gconf_get_client ();
-	profile = panel_gconf_get_profile ();
+	panel_widget_set_applet_expandable (
+		frame->priv->panel, GTK_WIDGET (frame), major, minor);
 
-	temp_key = panel_gconf_full_key (PANEL_GCONF_APPLETS, profile, gconf_key, "bonobo_iid");
-	applet_iid = gconf_client_get_string (client, temp_key, NULL);
+	frame->priv->has_handle = (flags & APPLET_HAS_HANDLE) != 0;
+}
 
-	panel_applet_frame_load (applet_iid, panel_widget, position, TRUE, gconf_key);
+static void
+panel_applet_frame_set_size_hints_from_any (PanelAppletFrame *frame,
+					    const CORBA_any  *any)
+{
+	CORBA_sequence_CORBA_long *seq;
+	int                       *size_hints;
+	int                        extra_size;
+	int                        i;
 
-	g_free (applet_iid);
+	seq = any->_value;
+
+	size_hints = g_new0 (int, seq->_length);
+
+	extra_size = 0;
+	if (frame->priv->has_handle)
+		extra_size = HANDLE_SIZE + 1;
+	
+	for (i = 0; i < seq->_length; i++)
+		size_hints [i] = seq->_buffer [i] + extra_size;
+	
+	panel_widget_set_applet_size_hints (frame->priv->panel,
+					    GTK_WIDGET (frame),
+					    size_hints,
+					    seq->_length);
+}
+
+static void
+panel_applet_frame_init_properties (PanelAppletFrame *frame)
+{
+	CORBA_any *any;
+
+	any = bonobo_pbclient_get_value (frame->priv->property_bag,
+					 PROPERTY_FLAGS,
+					 BONOBO_ARG_SHORT,
+					 NULL);
+	if (any) {
+		panel_applet_frame_set_flags_from_any (frame, any);
+		CORBA_free (any);
+	}
+	
+	any = bonobo_pbclient_get_value (frame->priv->property_bag,
+					 PROPERTY_SIZE_HINTS,
+					 TC_CORBA_sequence_CORBA_long,
+					 NULL);
+	if (any) {
+		panel_applet_frame_set_size_hints_from_any (frame, any);
+		CORBA_free (any);
+	}
 }
 
 static void
@@ -122,7 +202,17 @@ popup_handle_remove (BonoboUIComponent *uic,
 	info = frame->priv->applet_info;
 	frame->priv->applet_info = NULL;
 
-	panel_applet_clean (info, TRUE);
+	panel_profile_delete_object (info);
+}
+
+static void
+popup_handle_lock (BonoboUIComponent *uic,
+		   PanelAppletFrame  *frame,
+		   const char        *verb)
+{
+	panel_applet_toggle_locked (frame->priv->applet_info);
+
+	panel_applet_frame_sync_menu_state (frame);
 }
 
 static void
@@ -133,133 +223,137 @@ popup_handle_move (BonoboUIComponent *uic,
 	GtkWidget *widget;
 
 	g_return_if_fail (GTK_IS_WIDGET (frame));
-	g_return_if_fail (PANEL_IS_WIDGET (frame->priv->panel));
 
 	widget = GTK_WIDGET (frame);
-
+	
 	g_return_if_fail (PANEL_IS_WIDGET (widget->parent));
 
 	panel_widget_applet_drag_start (
-		frame->priv->panel, widget, PW_DRAG_OFF_CENTER);
+		PANEL_WIDGET (widget->parent), widget, PW_DRAG_OFF_CENTER, GDK_CURRENT_TIME);
 }
 
 static BonoboUIVerb popup_verbs [] = {
         BONOBO_UI_UNSAFE_VERB ("RemoveAppletFromPanel", popup_handle_remove),
+	BONOBO_UI_UNSAFE_VERB ("LockAppletToPanel",     popup_handle_lock),
         BONOBO_UI_UNSAFE_VERB ("MoveApplet",            popup_handle_move),
 
         BONOBO_UI_VERB_END
 };
 
-void
+
+static void
 panel_applet_frame_load (const gchar *iid,
 			 PanelWidget *panel,
+			 gboolean     locked,
 			 int          position,
 			 gboolean     exactpos,
-			 const char  *gconf_key)
+			 const char  *id)
 {
-	GtkWidget  *frame = NULL;
-	AppletInfo *info;
-	char       *real_key;
+	PanelAppletFrame *frame;
+	GtkWidget        *widget;
+	AppletInfo       *info;
 
 	g_return_if_fail (iid != NULL);
 	g_return_if_fail (panel != NULL);
+	g_return_if_fail (id != NULL);
 
-	if (gconf_key)
-		real_key = g_strdup (gconf_key);
-	else
-		real_key = gconf_unique_key ();
-
-	frame = panel_applet_frame_new (panel, iid, real_key);
-
-	if (!frame) {
-		g_free (real_key);
+	widget = panel_applet_frame_new (panel, iid, id);
+	if (!widget)
 		return;
+	
+	gtk_widget_show_all (widget);
+
+	info = panel_applet_register (widget, widget, NULL,
+				      panel, locked, position,
+				      exactpos, PANEL_OBJECT_BONOBO, id);
+
+	panel_widget_set_applet_size_constrained (panel, widget, TRUE);
+
+	frame = PANEL_APPLET_FRAME (widget);
+
+	panel_applet_frame_set_info (frame, info);
+	panel_applet_frame_sync_menu_state (frame);
+	panel_applet_frame_init_properties (frame);
+}
+
+void
+panel_applet_frame_load_from_gconf (PanelWidget *panel_widget,
+				    gboolean     locked,
+				    int          position,
+				    const char  *id)
+{
+	const char  *profile;
+	const char  *key;
+	char        *applet_iid;
+
+	g_return_if_fail (panel_widget != NULL);
+	g_return_if_fail (id != NULL);
+
+	profile = panel_profile_get_name ();
+
+	key = panel_gconf_full_key (PANEL_GCONF_APPLETS, profile, id, "bonobo_iid");
+	applet_iid = gconf_client_get_string (panel_gconf_get_client (), key, NULL);
+
+	panel_applet_frame_load (applet_iid, panel_widget, locked, position, TRUE, id);
+
+	g_free (applet_iid);
+}
+
+void
+panel_applet_frame_create (PanelToplevel *toplevel,
+			   int            position,
+			   const char    *iid)
+{
+	GConfClient *client;
+	const char  *profile;
+	const char  *key;
+	char        *id;
+
+	g_return_if_fail (iid != NULL);
+
+	client  = panel_gconf_get_client ();
+	profile = panel_profile_get_name ();
+
+	id = panel_profile_prepare_object (PANEL_OBJECT_BONOBO, toplevel, position, FALSE);
+
+	key = panel_gconf_full_key (PANEL_GCONF_APPLETS, profile, id, "bonobo_iid");
+	gconf_client_set_string (client, key, iid, NULL);
+
+	/* frees id */
+	panel_profile_add_to_list (PANEL_GCONF_APPLETS, id);
+}
+
+void
+panel_applet_frame_change_orientation (PanelAppletFrame *frame,
+				       PanelOrientation  orientation)
+{
+	CORBA_unsigned_short orient = 0;
+
+	if (orientation == frame->priv->orientation)
+		return;
+
+	frame->priv->orientation = orientation;
+
+	switch (orientation) {
+	case PANEL_ORIENTATION_TOP:
+		orient = GNOME_Vertigo_PANEL_ORIENT_DOWN;
+		break;
+	case PANEL_ORIENTATION_BOTTOM:
+		orient = GNOME_Vertigo_PANEL_ORIENT_UP;
+		break;
+	case PANEL_ORIENTATION_LEFT:
+		orient = GNOME_Vertigo_PANEL_ORIENT_RIGHT;
+		break;
+	case PANEL_ORIENTATION_RIGHT:
+		orient = GNOME_Vertigo_PANEL_ORIENT_LEFT;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
 	}
-	
-	gtk_widget_show_all (frame);
 
-	info = panel_applet_register (frame, frame, NULL, panel, position,
-				      exactpos, APPLET_BONOBO, real_key);
-
-	if (!info)
-		g_warning (_("Cannot register control widget\n"));
-
-	g_free (real_key);
-
-	panel_applet_frame_set_info (PANEL_APPLET_FRAME (frame), info);
-}
-
-static int
-panel_applet_frame_get_flags (PanelAppletFrame *frame)
-{
-	return bonobo_pbclient_get_short (
-			frame->priv->property_bag, "panel-applet-flags", NULL);
-}
-
-void
-panel_applet_frame_get_expand_flags (PanelAppletFrame *frame,
-				     gboolean         *expand_major,
-				     gboolean         *expand_minor)
-{
-	int flags;
-
-	flags = panel_applet_frame_get_flags (frame);
-
-	*expand_major = flags & APPLET_EXPAND_MAJOR;
-	*expand_minor = flags & APPLET_EXPAND_MINOR;
-}
-
-int 
-panel_applet_frame_get_size_hints (PanelAppletFrame  *frame,
-				   int              **size_hints)
-{
-	CORBA_sequence_CORBA_long *seq;
-	CORBA_any                 *value;
-	int                        retval = 0;
-	int                        extra_size = 0;
-	int                        i;
-
-
-	g_return_val_if_fail (PANEL_IS_APPLET_FRAME (frame), 0);
-	g_return_val_if_fail (size_hints != NULL, 0);
-
-	*size_hints = NULL;
-
-	value = bonobo_pbclient_get_value (frame->priv->property_bag,
-					   "panel-applet-size-hints",
-					   TC_CORBA_sequence_CORBA_long,
-					   NULL);
-	
-	if (value == NULL)
-		return retval;
-
-	seq = value->_value;
-
-	retval = seq->_length;
-	*size_hints = g_new (int, seq->_length);
-
-	extra_size = 0;
-	if (panel_applet_frame_get_flags (frame) & APPLET_HAS_HANDLE)
-		extra_size = HANDLE_SIZE + 1;
-	
-	for (i = 0; i < seq->_length; i++)
-		(*size_hints) [i] = seq->_buffer [i] + extra_size;
-	
-	CORBA_free (value);
-	
-	return retval;
-}
-
-void
-panel_applet_frame_change_orient (PanelAppletFrame *frame,
-				  PanelOrient       orient)
-{
-	if (orient == frame->priv->orient)
-		return;
-
-	frame->priv->orient = orient;
 	bonobo_pbclient_set_short (frame->priv->property_bag, 
-				   "panel-applet-orient",
+				   PROPERTY_ORIENT,
 				   orient,
 				   NULL);
 
@@ -270,8 +364,18 @@ void
 panel_applet_frame_change_size (PanelAppletFrame *frame,
 				PanelSize         size)
 {
+	/* Normalise the size to the constants defined in
+	 * the IDL.
+	 */
+	size = size <= PANEL_SIZE_XX_SMALL ? PANEL_SIZE_XX_SMALL :
+	       size <= PANEL_SIZE_X_SMALL  ? PANEL_SIZE_X_SMALL  :
+	       size <= PANEL_SIZE_SMALL    ? PANEL_SIZE_SMALL    :
+	       size <= PANEL_SIZE_MEDIUM   ? PANEL_SIZE_MEDIUM   :
+	       size <= PANEL_SIZE_LARGE    ? PANEL_SIZE_LARGE    :
+	       size <= PANEL_SIZE_X_LARGE  ? PANEL_SIZE_X_LARGE  : PANEL_SIZE_XX_LARGE;
+		 
 	bonobo_pbclient_set_short (frame->priv->property_bag, 
-				   "panel-applet-size",
+				   PROPERTY_SIZE,
 				   size,
 				   NULL);
 }
@@ -300,7 +404,7 @@ panel_applet_frame_change_background (PanelAppletFrame    *frame,
 			frame, PANEL_WIDGET (GTK_WIDGET (frame)->parent), type);
 
 	bonobo_pbclient_set_string (frame->priv->property_bag,
-				    "panel-applet-background",
+				    PROPERTY_BACKGROUND,
 				    bg_str, NULL);
 
 	g_free (bg_str);
@@ -347,20 +451,20 @@ panel_applet_frame_paint (GtkWidget    *widget,
 
 	frame = PANEL_APPLET_FRAME (widget);
 
-	if (!(panel_applet_frame_get_flags (frame) & APPLET_HAS_HANDLE))
+	if (!frame->priv->has_handle)
 		return;
   
 	if (GTK_WIDGET_DRAWABLE (widget)) {
-		GtkOrientation orient = 0;
+		GtkOrientation orientation = GTK_ORIENTATION_HORIZONTAL;
 
-		switch (frame->priv->orient) {
-		case PANEL_ORIENT_UP:
-		case PANEL_ORIENT_DOWN:
-			orient = GTK_ORIENTATION_HORIZONTAL;
+		switch (frame->priv->orientation) {
+		case PANEL_ORIENTATION_TOP:
+		case PANEL_ORIENTATION_BOTTOM:
+			orientation = GTK_ORIENTATION_HORIZONTAL;
 			break;
-		case PANEL_ORIENT_LEFT:
-		case PANEL_ORIENT_RIGHT:
-			orient = GTK_ORIENTATION_VERTICAL;
+		case PANEL_ORIENTATION_LEFT:
+		case PANEL_ORIENTATION_RIGHT:
+			orientation = GTK_ORIENTATION_VERTICAL;
 			break;
 		default:
 			g_assert_not_reached ();
@@ -376,7 +480,7 @@ panel_applet_frame_paint (GtkWidget    *widget,
                         frame->priv->handle_rect.y,
                         frame->priv->handle_rect.width,
                         frame->priv->handle_rect.height,
-                        orient);
+                        orientation);
 	}
 }
 
@@ -394,33 +498,6 @@ panel_applet_frame_expose (GtkWidget      *widget,
 	return FALSE;
 }
 
-static void
-panel_applet_frame_constrain_size (PanelAppletFrame *frame,
-				   GtkRequisition   *requisition)
-{
-	PanelWidget *panel;
-
-	g_return_if_fail (PANEL_IS_WIDGET (frame->priv->panel));
-
-	panel = frame->priv->panel;
-
-	switch (frame->priv->orient) {
-	case PANEL_ORIENT_UP:
-	case PANEL_ORIENT_DOWN:
-		if (requisition->height > panel->sz)
-			requisition->height = panel->sz;
-		break;
-	case PANEL_ORIENT_LEFT:
-	case PANEL_ORIENT_RIGHT:
-		if (requisition->width > panel->sz)
-			requisition->width = panel->sz;
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-}
-
 static void
 panel_applet_frame_size_request (GtkWidget      *widget,
 				 GtkRequisition *requisition)
@@ -432,9 +509,8 @@ panel_applet_frame_size_request (GtkWidget      *widget,
 	frame = PANEL_APPLET_FRAME (widget);
 	bin = GTK_BIN (widget);
 
-	if (!(panel_applet_frame_get_flags (frame) & APPLET_HAS_HANDLE)) {
+	if (!frame->priv->has_handle) {
 		GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
-		panel_applet_frame_constrain_size (frame, requisition);
 		return;
 	}
   
@@ -451,21 +527,19 @@ panel_applet_frame_size_request (GtkWidget      *widget,
 	requisition->width += GTK_CONTAINER (widget)->border_width;
 	requisition->height += GTK_CONTAINER (widget)->border_width;
 
-	switch (frame->priv->orient) {
-	case PANEL_ORIENT_UP:
-	case PANEL_ORIENT_DOWN:
+	switch (frame->priv->orientation) {
+	case PANEL_ORIENTATION_TOP:
+	case PANEL_ORIENTATION_BOTTOM:
 		requisition->width += HANDLE_SIZE;
 		break;
-	case PANEL_ORIENT_LEFT:
-	case PANEL_ORIENT_RIGHT:
+	case PANEL_ORIENTATION_LEFT:
+	case PANEL_ORIENTATION_RIGHT:
 		requisition->height += HANDLE_SIZE;
 		break;
 	default:
 		g_assert_not_reached ();
 		break;
 	}
-
-	panel_applet_frame_constrain_size (frame, requisition);
 }
 
 static void
@@ -479,7 +553,7 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 	frame = PANEL_APPLET_FRAME (widget);
 	bin = GTK_BIN (widget);
 
-	if (!(panel_applet_frame_get_flags (frame) & APPLET_HAS_HANDLE)) {
+	if (!frame->priv->has_handle) {
 		GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
 		return;
 	}
@@ -489,9 +563,9 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 	frame->priv->handle_rect.x = 0;
 	frame->priv->handle_rect.y = 0;
 
-	switch (frame->priv->orient) {
-	case PANEL_ORIENT_UP:
-	case PANEL_ORIENT_DOWN:
+	switch (frame->priv->orientation) {
+	case PANEL_ORIENTATION_TOP:
+	case PANEL_ORIENTATION_BOTTOM:
 		frame->priv->handle_rect.width  = HANDLE_SIZE;
 		frame->priv->handle_rect.height = allocation->height;
 
@@ -500,8 +574,8 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 		new_allocation.width  = allocation->width - HANDLE_SIZE;
 		new_allocation.height = allocation->height;
 		break;
-	case PANEL_ORIENT_LEFT:
-	case PANEL_ORIENT_RIGHT:
+	case PANEL_ORIENTATION_LEFT:
+	case PANEL_ORIENTATION_RIGHT:
 		frame->priv->handle_rect.width  = allocation->width;
 		frame->priv->handle_rect.height = HANDLE_SIZE;
 
@@ -564,7 +638,7 @@ panel_applet_frame_button_changed (GtkWidget      *widget,
 
 	frame = PANEL_APPLET_FRAME (widget);
 
-	if (!(panel_applet_frame_get_flags (frame) & APPLET_HAS_HANDLE))
+	if (!frame->priv->has_handle)
 		return handled;
 
 	if (event->window != widget->window)
@@ -577,7 +651,8 @@ panel_applet_frame_button_changed (GtkWidget      *widget,
 			if (event->type == GDK_BUTTON_PRESS ||
 			    event->type == GDK_2BUTTON_PRESS) {
 				panel_widget_applet_drag_start (
-					frame->priv->panel, GTK_WIDGET (frame), PW_DRAG_OFF_CURSOR);
+					frame->priv->panel, GTK_WIDGET (frame),
+					PW_DRAG_OFF_CURSOR, event->time);
 				handled = TRUE;
 			} else if (event->type == GDK_BUTTON_RELEASE) {
 				panel_widget_applet_drag_end (frame->priv->panel);
@@ -632,25 +707,33 @@ panel_applet_frame_reload_response (GtkWidget        *dialog,
 	if (response == GTK_RESPONSE_YES) {
 		PanelWidget *panel;
 		char        *iid;
-		char        *gconf_key = NULL;
+		char        *id = NULL;
 		int          position = -1;
+		gboolean     locked = FALSE;
 
 		panel = frame->priv->panel;
 		iid   = g_strdup (frame->priv->iid);
 
 		if (info) {
-			gconf_key = g_strdup (info->gconf_key);
+			id = g_strdup (info->id);
 			position  = panel_applet_get_position (info);
-			panel_applet_clean (info, FALSE);
+			locked = panel_widget_get_applet_locked (panel, info->widget);
+			panel_applet_clean (info);
 		}
 
-		panel_applet_frame_load (iid, panel, position, TRUE, gconf_key);
+		panel_applet_frame_load (iid, panel, locked,
+					 position, TRUE, id);
 
 		g_free (iid);
-		g_free (gconf_key);
+		g_free (id);
 
-	} else if (info)
-		panel_applet_clean (info, TRUE);
+	} else if (info) {
+		/* if we can't write to applets list we can't really delete
+		   it, so we'll just ignore this.  FIXME: handle this
+		   more correctly I suppose. */
+		if (panel_profile_list_is_writable (PANEL_GCONF_APPLETS))
+			panel_profile_delete_object (info);
+	}
 
 	g_object_unref (frame);
 	gtk_widget_destroy (dialog);
@@ -764,25 +847,32 @@ static void
 panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 				   CORBA_Environment *ev,
 				   const char        *iid,
-				   const char        *gconf_key,
+				   const char        *id,
 				   GtkWindow         *panel)
 {
 	GtkWidget *dialog;
-	char      *error;
 	int        response;
 
-	error = bonobo_exception_get_text (ev);
-
-	dialog = gtk_message_dialog_new (
-				NULL, 0,
-				GTK_MESSAGE_QUESTION,
-				GTK_BUTTONS_NONE,
-				_("The panel encountered a problem while loading \"%s\"\n"
-				  "Details: %s\n\n"
-				  "Do you want to delete the applet from your configuration?"),
-				iid, error);
-
-	g_free (error);
+	if (ev != NULL) {
+		char *error = bonobo_exception_get_text (ev);
+		dialog = gtk_message_dialog_new (
+					NULL, 0,
+					GTK_MESSAGE_QUESTION,
+					GTK_BUTTONS_NONE,
+					_("The panel encountered a problem while loading \"%s\"\n"
+					  "Details: %s\n\n"
+					  "Do you want to delete the applet from your configuration?"),
+					iid, error);
+		g_free (error);
+	} else {
+		dialog = gtk_message_dialog_new (
+					NULL, 0,
+					GTK_MESSAGE_QUESTION,
+					GTK_BUTTONS_NONE,
+					_("The panel encountered a problem while loading \"%s\"\n\n"
+					  "Do you want to delete the applet from your configuration?"),
+					iid);
+	}
 
 	register_stock_item ();
 
@@ -799,8 +889,13 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 
 	gtk_widget_destroy (dialog);
 
-	if (response == GTK_RESPONSE_OK)
-		panel_applet_clean_gconf (APPLET_BONOBO, gconf_key, TRUE);
+	if (response == GTK_RESPONSE_OK) {
+		/* if we can't write to applets list we can't really delete
+		   it, so we'll just ignore this.  There's nothing we can
+		   do about it. */
+		if (panel_profile_list_is_writable (PANEL_GCONF_APPLETS))
+			panel_profile_remove_from_list (PANEL_GCONF_APPLETS, id);
+	}
 }
 
 static void
@@ -831,9 +926,10 @@ panel_applet_frame_instance_init (PanelAppletFrame      *frame,
 	frame->priv->property_bag     = CORBA_OBJECT_NIL;
 	frame->priv->ui_component     = NULL;
 	frame->priv->panel            = NULL;
-	frame->priv->orient           = PANEL_ORIENT_UP;
+	frame->priv->orientation      = PANEL_ORIENTATION_TOP;
 	frame->priv->applet_info      = NULL;
 	frame->priv->moving_focus_out = FALSE;
+	frame->priv->has_handle       = FALSE;
 }
 
 GType
@@ -889,24 +985,24 @@ static G_CONST_RETURN char *
 panel_applet_frame_get_orient_string (PanelAppletFrame *frame,
 				      PanelWidget      *panel)
 {
-	PanelOrient  orient;
-	const char  *retval = NULL;
+	PanelOrientation  orientation;
+	const char       *retval = NULL;
 
-	orient = panel_widget_get_applet_orient (panel);
+	orientation = panel_widget_get_applet_orientation (panel);
 
-	switch (orient) {
-	case PANEL_ORIENT_UP:
-		retval = "up";
-		break;
-	case PANEL_ORIENT_DOWN:
+	switch (orientation) {
+	case PANEL_ORIENTATION_TOP:
 		retval = "down";
 		break;
-	case PANEL_ORIENT_LEFT:
-		retval = "left";
+	case PANEL_ORIENTATION_BOTTOM:
+		retval = "up";
 		break;
-	case PANEL_ORIENT_RIGHT:
+	case PANEL_ORIENTATION_LEFT:
 		retval = "right";
 		break;
+	case PANEL_ORIENTATION_RIGHT:
+		retval = "left";
+		break;
 	default:
 		g_assert_not_reached ();
 		break;
@@ -921,32 +1017,20 @@ panel_applet_frame_get_size_string (PanelAppletFrame *frame,
 {
 	const char *retval = NULL;
 
-	switch (panel->sz) {
-	case PANEL_SIZE_XX_SMALL:
+	if (panel->sz <= PANEL_SIZE_XX_SMALL)
 		retval = "xx-small";
-		break;
-	case PANEL_SIZE_X_SMALL:
+	else if (panel->sz <= PANEL_SIZE_X_SMALL)
 		retval = "x-small";
-		break;
-	case PANEL_SIZE_SMALL:
+	else if (panel->sz <= PANEL_SIZE_SMALL)
 		retval = "small";
-		break;
-	case PANEL_SIZE_MEDIUM:
+	else if (panel->sz <= PANEL_SIZE_MEDIUM)
 		retval = "medium";
-		break;
-	case PANEL_SIZE_LARGE:
+	else if (panel->sz <= PANEL_SIZE_LARGE)
 		retval = "large";
-		break;
-	case PANEL_SIZE_X_LARGE:
+	else if (panel->sz <= PANEL_SIZE_X_LARGE)
 		retval = "x-large";
-		break;
-	case PANEL_SIZE_XX_LARGE:
+	else
 		retval = "xx-large";
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
 
 	return retval;
 }
@@ -955,7 +1039,7 @@ static char *
 panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
 				      PanelWidget      *panel,
 				      const char       *iid,
-				      const char       *gconf_key)
+				      const char       *id)
 {
 	char *retval;
 	char *bg_str;
@@ -966,7 +1050,7 @@ panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
 	retval = g_strdup_printf (
 			"%s!prefs_key=/apps/panel/profiles/%s/applets/%s/prefs;"
 			"background=%s;orient=%s;size=%s",
-			iid, panel_gconf_get_profile (), gconf_key, bg_str,
+			iid, panel_profile_get_name (), id, bg_str,
 			panel_applet_frame_get_orient_string (frame, panel),
 			panel_applet_frame_get_size_string (frame, panel));
 
@@ -975,11 +1059,25 @@ panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
 	return retval;
 }
 
+static void
+panel_applet_frame_event_listener (BonoboListener    *listener,
+				   const char        *event,
+				   const CORBA_any   *any,
+				   CORBA_Environment *ev,
+				   PanelAppletFrame  *frame)
+{
+	if (!strcmp (event, "Bonobo/Property:change:" PROPERTY_FLAGS))
+		panel_applet_frame_set_flags_from_any (frame, any);
+
+	else if (!strcmp (event, "Bonobo/Property:change:" PROPERTY_SIZE_HINTS))
+		panel_applet_frame_set_size_hints_from_any (frame, any);
+}
+
 GtkWidget *
 panel_applet_frame_construct (PanelAppletFrame *frame,
 			      PanelWidget      *panel,
 			      const char       *iid,
-			      const char       *gconf_key)
+			      const char       *id)
 {
 	BonoboControlFrame    *control_frame;
 	Bonobo_Control         control;
@@ -990,7 +1088,7 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 
 	frame->priv->panel = panel;
 
-	moniker = panel_applet_frame_construct_moniker (frame, panel, iid, gconf_key);
+	moniker = panel_applet_frame_construct_moniker (frame, panel, iid, id);
 
 	/* FIXME: this should really use bonobo_get_object_async */
 	CORBA_exception_init (&ev);
@@ -1000,44 +1098,66 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 
 	g_free (moniker);
 
-	if (BONOBO_EX (&ev)) {
+	/* according to the source of bonobo control == NULL && no
+	   exception can happen, so handle it */
+	if (BONOBO_EX (&ev) || control == CORBA_OBJECT_NIL) {
 		panel_applet_frame_loading_failed (
-			frame, &ev, iid, gconf_key, GTK_WINDOW (panel->panel_parent));
+			frame, &ev, iid, id, GTK_WINDOW (panel->toplevel));
 		CORBA_exception_free (&ev);
 		return NULL;
 	}
 
 	frame->priv->iid = g_strdup (iid);
 
-	cnx_status = ORBit_small_get_connection_status (control);
-	if (cnx_status != ORBIT_CONNECTION_IN_PROC)
-		g_signal_connect_object (
-			ORBit_small_get_connection (control),
-			"broken",
-			G_CALLBACK (panel_applet_frame_cnx_broken),
-			frame,
-			G_CONNECT_SWAPPED);
-	
-	CORBA_exception_free (&ev);
-
 	widget = bonobo_widget_new_control_from_objref (control, CORBA_OBJECT_NIL);
 
 	bonobo_object_release_unref (control, NULL);
 
 	if (!widget) {
+		CORBA_exception_free (&ev);
+		panel_applet_frame_loading_failed (
+			frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
 		g_warning (G_STRLOC ": failed to load applet %s", iid);
 		return NULL;
 	}
 
 	control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (widget));
+	if (control_frame == NULL) {
+		CORBA_exception_free (&ev);
+		panel_applet_frame_loading_failed (
+			frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
+		g_warning (G_STRLOC ": failed to load applet %s (can't get control frame)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
+		return NULL;
+	}
 
 	frame->priv->property_bag = 
-		bonobo_control_frame_get_control_property_bag (control_frame, NULL);
-
-	frame->priv->ui_component =
-		bonobo_control_frame_get_popup_component (control_frame, NULL);
-	if (!frame->priv->ui_component)
+		bonobo_control_frame_get_control_property_bag (control_frame, &ev);
+	if (frame->priv->property_bag == NULL || BONOBO_EX (&ev)) {
+		panel_applet_frame_loading_failed (
+			frame, &ev, iid, id, GTK_WINDOW (panel->toplevel));
+		CORBA_exception_free (&ev);
+		g_warning (G_STRLOC ": failed to load applet %s (can't get property bag)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
 		return NULL;
+	}
+
+	bonobo_event_source_client_add_listener (frame->priv->property_bag,
+						 (BonoboListenerCallbackFn) panel_applet_frame_event_listener,
+						 "Bonobo/Property:change:panel-applet",
+						 NULL,
+						 frame);
+	
+	frame->priv->ui_component =
+		bonobo_control_frame_get_popup_component (control_frame, &ev);
+	if (frame->priv->ui_component == NULL || BONOBO_EX (&ev)) {
+		panel_applet_frame_loading_failed (
+			frame, &ev, iid, id, GTK_WINDOW (panel->toplevel));
+		CORBA_exception_free (&ev);
+		g_warning (G_STRLOC ": failed to load applet %s (can't get popup component)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
+		return NULL;
+	}
 
 	bonobo_ui_util_set_ui (frame->priv->ui_component, DATADIR,
 			       "GNOME_Panel_Popup.xml", "panel", NULL);
@@ -1046,10 +1166,47 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 		frame->priv->ui_component, popup_verbs, frame);
 
 	control = bonobo_control_frame_get_control (control_frame);
-	if (!control)
+	if (!control) {
+		CORBA_exception_free (&ev);
+		panel_applet_frame_loading_failed (
+			frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
+		g_warning (G_STRLOC ": failed to load applet %s (can't get control)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
 		return NULL;
+	}
 
 	frame->priv->applet_shell = panel_applet_frame_get_applet_shell (control);
+	if (frame->priv->applet_shell == CORBA_OBJECT_NIL) {
+		CORBA_exception_free (&ev);
+		panel_applet_frame_loading_failed (
+			frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
+		g_warning (G_STRLOC ": failed to load applet %s (can't get applet shell)", iid);
+		gtk_object_sink (GTK_OBJECT (widget));
+		return NULL;
+	}
+
+	CORBA_exception_free (&ev);
+
+	cnx_status = ORBit_small_get_connection_status (control);
+	if (cnx_status != ORBIT_CONNECTION_IN_PROC) {
+		ORBitConnection *cnx = ORBit_small_get_connection_ref (control);
+		if (cnx == NULL) {
+			CORBA_exception_free (&ev);
+			panel_applet_frame_loading_failed (
+				frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
+			g_warning (G_STRLOC ": failed to load applet %s (lost connection)", iid);
+			gtk_object_sink (GTK_OBJECT (widget));
+			return NULL;
+		}
+		g_signal_connect_object (
+			cnx,
+			"broken",
+			G_CALLBACK (panel_applet_frame_cnx_broken),
+			frame,
+			G_CONNECT_SWAPPED);
+		ORBit_small_connection_unref (cnx);
+	}
+	
 
 	gtk_container_add (GTK_CONTAINER (frame), widget);
 
@@ -1059,18 +1216,36 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 GtkWidget *
 panel_applet_frame_new (PanelWidget *panel,
 			const char  *iid,
-			const char  *gconf_key)
+			const char  *id)
 {
 	PanelAppletFrame *frame;
 
-	g_return_val_if_fail (iid != NULL && gconf_key != NULL, NULL);
+	g_return_val_if_fail (iid != NULL && id != NULL, NULL);
 
 	frame = g_object_new (PANEL_TYPE_APPLET_FRAME, NULL);
 
-	if (!panel_applet_frame_construct (frame, panel, iid, gconf_key)) {
-		gtk_object_sink (GTK_OBJECT (frame));
+	if (!panel_applet_frame_construct (frame, panel, iid, id)) {
+		//gtk_object_sink (GTK_OBJECT (frame));
 		return NULL;
 	}
 
 	return GTK_WIDGET (frame);
 }
+
+PanelWidget *
+panel_applet_frame_get_panel (PanelAppletFrame *frame)
+{
+	g_return_val_if_fail (PANEL_IS_APPLET_FRAME (frame), NULL);
+
+	return frame->priv->panel;
+}
+
+void
+panel_applet_frame_set_panel (PanelAppletFrame *frame,
+			      PanelWidget      *panel)
+{
+	g_return_if_fail (PANEL_IS_APPLET_FRAME (frame));
+	g_return_if_fail (PANEL_IS_WIDGET (panel));
+
+	frame->priv->panel = panel;
+}
