@@ -57,6 +57,7 @@ typedef struct {
 } ToplevelLocationChange;
 
 typedef const char *(*PanelProfileGetIdFunc)   (gpointer           object);
+typedef gboolean    (*PanelProfileOnLoadQueue) (const char        *id);
 typedef void        (*PanelProfileLoadFunc)    (GConfClient       *client,
 						const char        *profile_dir, 
 						PanelGConfKeyType  type,
@@ -277,6 +278,8 @@ panel_profile_find_new_id (PanelGConfKeyType type)
 			}
 	}
 
+	g_assert (retval != NULL);
+
 	for (l = existing_ids; l; l = l->next)
 		g_free (l->data);
 	g_slist_free (existing_ids);
@@ -338,6 +341,17 @@ panel_profile_get_toplevel_key (PanelToplevel *toplevel,
 	return panel_gconf_full_key (PANEL_GCONF_TOPLEVELS, current_profile, id, key);
 }
 
+#define TOPLEVEL_IS_WRITABLE_FUNC(k, p, s)                            \
+	gboolean                                                      \
+	panel_profile_is_writable_##p##_##s (PanelToplevel *toplevel) \
+	{                                                             \
+		GConfClient *client;                                  \
+		const char  *key;                                     \
+		client = panel_gconf_get_client ();                   \
+		key = panel_profile_get_toplevel_key (toplevel, k);   \
+		return gconf_client_key_is_writable (client, key, NULL); \
+	}
+
 void
 panel_profile_set_background_type (PanelToplevel       *toplevel,
 				   PanelBackgroundType  background_type)
@@ -375,6 +389,8 @@ panel_profile_get_background_type (PanelToplevel *toplevel)
 	return background_type;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("background/type", background, type)
+
 void
 panel_profile_set_background_color (PanelToplevel *toplevel,
 				    PanelColor    *color)
@@ -405,6 +421,8 @@ panel_profile_get_background_color (PanelToplevel *toplevel,
 	color->alpha     = opacity;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("background/color", background, color)
+
 void
 panel_profile_set_background_pango_color (PanelToplevel *toplevel,
 					  PangoColor    *pango_color)
@@ -475,6 +493,8 @@ panel_profile_get_background_opacity (PanelToplevel *toplevel)
 	return opacity;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("background/opacity", background, opacity)
+
 void
 panel_profile_set_background_image (PanelToplevel *toplevel,
 				    const char    *image)
@@ -507,6 +527,8 @@ panel_profile_get_background_image (PanelToplevel *toplevel)
 	return retval;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("background/image", background, image)
+
 void
 panel_profile_set_toplevel_name (PanelToplevel *toplevel,
 				 const char    *name)
@@ -539,6 +561,8 @@ panel_profile_get_toplevel_name (PanelToplevel *toplevel)
 	return retval;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("name", toplevel, name)
+
 void
 panel_profile_set_toplevel_orientation (PanelToplevel    *toplevel,
 					PanelOrientation  orientation)
@@ -568,13 +592,16 @@ panel_profile_get_toplevel_orientation (PanelToplevel *toplevel)
 	key = panel_profile_get_toplevel_key (toplevel, "orientation");
 	str = gconf_client_get_string (client, key, NULL);
 
-	panel_profile_map_orientation_string (str, &orientation);
+	if (!panel_profile_map_orientation_string (str, &orientation))
+	    orientation = panel_toplevel_get_orientation (toplevel);
 
 	g_free (str);
 
 	return orientation;
 }
 
+TOPLEVEL_IS_WRITABLE_FUNC ("orientation", toplevel, orientation)
+
 #define TOPLEVEL_GET_SET_FUNCS(k, p, t, s, a)                         \
 	void                                                          \
 	panel_profile_set_##p##_##s (PanelToplevel *toplevel, a s)    \
@@ -595,7 +622,8 @@ panel_profile_get_toplevel_orientation (PanelToplevel *toplevel)
 		key = panel_profile_get_toplevel_key (toplevel, k);   \
 		retval = gconf_client_get_##t (client, key, NULL);    \
 		return retval;                                        \
-	}
+	}                                                             \
+        TOPLEVEL_IS_WRITABLE_FUNC(k, p, s)
 
 TOPLEVEL_GET_SET_FUNCS ("size",               toplevel,   int,  size,           int)
 TOPLEVEL_GET_SET_FUNCS ("expand",             toplevel,   bool, expand,         gboolean)
@@ -805,7 +833,7 @@ panel_profile_queue_toplevel_location_change (PanelToplevel          *toplevel,
 			panel_profile_get_toplevel_key (toplevel, "y_centered"),
 			change->y_centered);
 
-	if (change->orientation)
+	if (change->orientation_changed)
 		gconf_change_set_set_string (
 			queued_changes,
 			panel_profile_get_toplevel_key (toplevel, "orientation"),
@@ -1174,6 +1202,21 @@ panel_profile_remove_from_list (PanelGConfKeyType  type,
 	g_slist_free (list);
 }
 
+gboolean
+panel_profile_list_is_writable (PanelGConfKeyType type)
+{
+	GConfClient *client;
+	const char  *key;
+	const char  *id_list;
+
+	client = panel_gconf_get_client ();
+
+	id_list = panel_gconf_key_type_to_id_list (type);
+
+	key = panel_gconf_general_key (current_profile, id_list);
+	return gconf_client_key_is_writable (client, key, NULL);
+}
+
 void
 panel_profile_create_toplevel (void)
 {
@@ -1265,7 +1308,7 @@ get_toplevel_screen (GConfClient   *client,
 	return gdk_display_get_screen (display, screen_n);
 }
 
-void
+PanelToplevel *
 panel_profile_load_toplevel (GConfClient       *client,
 			     const char        *profile_dir,
 			     PanelGConfKeyType  type,
@@ -1278,8 +1321,10 @@ panel_profile_load_toplevel (GConfClient       *client,
 	char          *toplevel_dir;
 	guint          notify_id;
 
-	if (!toplevel_id || !toplevel_id [0])
-		return;
+	if (!toplevel_id || !toplevel_id [0]) {
+		g_free (toplevel_id);
+		return NULL;
+	}
 
 	toplevel_dir = g_strdup_printf ("%s/toplevels/%s", profile_dir, toplevel_id);
 
@@ -1287,16 +1332,23 @@ panel_profile_load_toplevel (GConfClient       *client,
 		panel_gconf_associate_schemas_in_dir (
 			client, toplevel_dir, PANEL_SCHEMAS_DIR "/toplevels");
 
-	if (!(screen = get_toplevel_screen (client, toplevel_dir))) {
-		g_free (toplevel_dir);
-		return;
-	}
-
 	gconf_client_add_dir (client,
 			      toplevel_dir,
-			      GCONF_CLIENT_PRELOAD_RECURSIVE,
+			      GCONF_CLIENT_PRELOAD_ONELEVEL,
 			      NULL);
 
+	key = panel_gconf_sprintf ("%s/background", toplevel_dir);
+	gconf_client_add_dir (client,
+			      key,
+			      GCONF_CLIENT_PRELOAD_ONELEVEL,
+			      NULL);
+
+	if (!(screen = get_toplevel_screen (client, toplevel_dir))) {
+		g_free (toplevel_id);
+		g_free (toplevel_dir);
+		return NULL;
+	}
+
 	toplevel = g_object_new (PANEL_TYPE_TOPLEVEL,
 				 "screen", screen,
 				 NULL);
@@ -1409,7 +1461,21 @@ panel_profile_load_toplevel (GConfClient       *client,
 	g_free (toplevel_dir);
 
 	panel_setup (toplevel);
-	gtk_widget_show (GTK_WIDGET (toplevel));
+
+	return toplevel;
+}
+
+static void
+panel_profile_load_and_show_toplevel (GConfClient       *client,
+				      const char        *profile_dir,
+				      PanelGConfKeyType  type,
+				      char              *toplevel_id)
+{
+	PanelToplevel *toplevel;
+
+	toplevel = panel_profile_load_toplevel (client, profile_dir, type, toplevel_id);
+	if (toplevel)
+		gtk_widget_show (GTK_WIDGET (toplevel));
 }
 
 static void
@@ -1426,7 +1492,8 @@ panel_profile_destroy_toplevel (const char *id)
 char *
 panel_profile_prepare_object_with_id (PanelObjectType  object_type,
 				      const char      *toplevel_id,
-				      int              position)
+				      int              position,
+				      gboolean         right_stick)
 {
 	PanelGConfKeyType  key_type;
 	GConfClient       *client;
@@ -1458,6 +1525,9 @@ panel_profile_prepare_object_with_id (PanelObjectType  object_type,
 	key = panel_gconf_full_key (key_type, current_profile, id, "position");
 	gconf_client_set_int (client, key, position, NULL);
 
+	key = panel_gconf_full_key (key_type, current_profile, id, "panel_right_stick");
+	gconf_client_set_bool (client, key, right_stick, NULL);
+
 	g_free (dir);
 
 	return id;
@@ -1466,11 +1536,13 @@ panel_profile_prepare_object_with_id (PanelObjectType  object_type,
 char *
 panel_profile_prepare_object (PanelObjectType  object_type,
 			      PanelToplevel   *toplevel,
-			      int              position)
+			      int              position,
+			      gboolean         right_stick)
 {
 	return panel_profile_prepare_object_with_id (object_type,
 						     panel_profile_get_toplevel_id (toplevel),
-						     position);
+						     position,
+						     right_stick);
 }
 
 void
@@ -1492,15 +1564,23 @@ panel_profile_load_object (GConfClient       *client,
 			   PanelGConfKeyType  type,
 			   char              *id)
 {
-	PanelToplevel   *toplevel;
 	PanelObjectType  object_type;
+	char            *object_dir;
 	const char      *key;
 	char            *type_string;
 	char            *toplevel_id;
 	int              position;
 	gboolean         right_stick;
+	gboolean         locked;
 
-	key = panel_gconf_full_key (type, current_profile, id, "object_type");
+	object_dir = g_strdup_printf ("%s/%s/%s",
+				      profile_dir,
+				      type == PANEL_GCONF_OBJECTS ? "objects" : "applets",
+				      id);
+
+	gconf_client_add_dir (client, object_dir, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+
+	key = panel_gconf_sprintf ("%s/object_type", object_dir);
 	type_string = gconf_client_get_string (client, key, NULL);
         
 	if (!panel_profile_map_object_type_string (type_string, &object_type)) {
@@ -1510,19 +1590,26 @@ panel_profile_load_object (GConfClient       *client,
 	
 	g_free (type_string);
 
-	key = panel_gconf_full_key (type, current_profile, id, "position");
+	key = panel_gconf_sprintf ("%s/position", object_dir);
 	position = gconf_client_get_int (client, key, NULL);
 	
-	key = panel_gconf_full_key (type, current_profile, id, "toplevel_id");
+	key = panel_gconf_sprintf ("%s/toplevel_id", object_dir);
 	toplevel_id = gconf_client_get_string (client, key, NULL);
 
-	toplevel = panel_profile_get_toplevel_by_id (toplevel_id);
-	g_free (toplevel_id);
-
-	key = panel_gconf_full_key (type, current_profile, id, "panel_right_stick");
+	key = panel_gconf_sprintf ("%s/panel_right_stick", object_dir);
 	right_stick = gconf_client_get_bool (client, key, NULL);
 
-	panel_applet_queue_applet_to_load (id, object_type, toplevel, position, right_stick);
+	key = panel_gconf_sprintf ("%s/locked", object_dir);
+	locked = gconf_client_get_bool (client, key, NULL);
+
+	g_free (object_dir);
+
+	panel_applet_queue_applet_to_load (id,
+					   object_type,
+					   toplevel_id,
+					   position,
+					   right_stick,
+					   locked);
 }
 
 static void
@@ -1561,6 +1648,10 @@ panel_profile_delete_dir (GConfClient       *client,
 
 	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/%s/%s",
 				   current_profile, type_str, id);
+
+	if (type == PANEL_GCONF_TOPLEVELS)
+	    gconf_client_remove_dir (client, key, NULL);
+
 	panel_gconf_clean_dir (client, key);
 }
 
@@ -1607,12 +1698,13 @@ panel_profile_id_exists (GSList     *id_list,
 }
 
 static void
-panel_profile_load_added_ids (GConfClient           *client,
-			      PanelGConfKeyType      type,
-			      GSList                *list,
-			      GSList                *id_list,
-			      PanelProfileGetIdFunc  get_id_func,
-			      PanelProfileLoadFunc   load_handler)
+panel_profile_load_added_ids (GConfClient            *client,
+			      PanelGConfKeyType       type,
+			      GSList                 *list,
+			      GSList                 *id_list,
+			      PanelProfileGetIdFunc   get_id_func,
+			      PanelProfileLoadFunc    load_handler,
+			      PanelProfileOnLoadQueue on_load_queue)
 {
 	GSList *added_ids = NULL;
 	GSList *l;
@@ -1621,7 +1713,8 @@ panel_profile_load_added_ids (GConfClient           *client,
 	for (l = id_list; l; l = l->next) {
 		const char *id = gconf_value_get_string (l->data);
 
-		if (!panel_profile_object_exists (list, id, get_id_func))
+		if (!panel_profile_object_exists (list, id, get_id_func) &&
+		    (on_load_queue == NULL || !on_load_queue (id)))
 			added_ids = g_slist_prepend (added_ids, g_strdup (id));
 	}
 
@@ -1699,7 +1792,8 @@ panel_profile_toplevel_id_list_notify (GConfClient *client,
 				      existing_toplevels,
 				      toplevel_ids,
 				      (PanelProfileGetIdFunc) panel_profile_get_toplevel_id,
-				      (PanelProfileLoadFunc) panel_profile_load_toplevel);
+				      (PanelProfileLoadFunc) panel_profile_load_and_show_toplevel,
+				      (PanelProfileOnLoadQueue) NULL);
 
 	panel_profile_delete_removed_ids (client,
 					  PANEL_GCONF_TOPLEVELS,
@@ -1747,7 +1841,8 @@ panel_profile_object_id_list_notify (GConfClient *client,
 				      sublist,
 				      object_ids,
 				      (PanelProfileGetIdFunc) panel_applet_get_id,
-				      (PanelProfileLoadFunc) panel_profile_load_object);
+				      (PanelProfileLoadFunc) panel_profile_load_object,
+				      (PanelProfileOnLoadQueue) panel_applet_on_load_queue);
 
 	panel_profile_delete_removed_ids (client,
 					  type,
@@ -1895,6 +1990,7 @@ panel_profile_copy_defaults_for_screen (GConfClient       *client,
 		char *new_id;
 		char *src_dir;
 		char *dest_dir;
+		char *schema_key;
 
 		new_id = g_strdup_printf ("%s_screen%d", default_id, screen_n);
 
@@ -1903,8 +1999,10 @@ panel_profile_copy_defaults_for_screen (GConfClient       *client,
 
 		panel_gconf_copy_dir (client, src_dir, dest_dir);
 
-		key = panel_gconf_sprintf (PANEL_SCHEMAS_DIR "/%s", schemas_str);
-		panel_gconf_associate_schemas_in_dir (client, dest_dir, key);
+		schema_key = g_strdup_printf (PANEL_SCHEMAS_DIR "/%s", 
+			                      schemas_str);
+		panel_gconf_associate_schemas_in_dir (client, dest_dir, schema_key);
+		g_free (schema_key);
 
 		new_ids = g_slist_prepend (new_ids, new_id);
 
@@ -1980,6 +2078,26 @@ panel_profile_copy_default_objects_for_screen (GConfClient       *client,
 	panel_profile_append_new_ids (client, type, new_objects);
 }
 
+/* FIXME:
+ *   We might want to do something more sophisticated like hardcode
+ *   the default panel setup as the fallback panels.
+ */
+static GSList *
+panel_profile_create_fallback_toplevel_list (GConfClient *client,
+					     const char  *profile_dir)
+{
+	char *id;
+	char *dir;
+
+	id = panel_profile_find_new_id (PANEL_GCONF_TOPLEVELS);
+
+	dir = g_strdup_printf ("%s/toplevels/%s", profile_dir, id);
+	panel_gconf_associate_schemas_in_dir (client, dir, PANEL_SCHEMAS_DIR "/toplevels");
+	g_free (dir);
+
+	return g_slist_prepend (NULL, id);
+}
+
 static void
 panel_profile_load_defaults_on_screen (GConfClient *client,
 				       const char  *profile_dir,
@@ -1992,6 +2110,11 @@ panel_profile_load_defaults_on_screen (GConfClient *client,
 
 	new_toplevels = panel_profile_copy_defaults_for_screen (
 				client, profile_dir, screen_n, PANEL_GCONF_TOPLEVELS);
+	if (!new_toplevels) {
+		g_warning ("Failed to load default panel configuration. panel-default-setup.entries "
+			   "hasn't been installed using gconftool-2 --load ?\n");
+		new_toplevels = panel_profile_create_fallback_toplevel_list (client, profile_dir);
+	}
 
 	for (l = new_toplevels; l; l = l->next) {
 		char       *toplevel_id = l->data;
@@ -2042,9 +2165,6 @@ panel_profile_ensure_toplevel_per_screen (GConfClient *client,
 
 	for (l = empty_screens; l; l = l->next)
 		panel_profile_load_defaults_on_screen (client, profile_dir, l->data);
-		/* FIXME: if we fail to load the defaults for whatever reason we
-		 *        should load a fallback panel.
-		 */
 
 	g_slist_free (empty_screens);
 }
@@ -2069,15 +2189,15 @@ panel_profile_load (char *profile_name)
 	if (!gconf_client_dir_exists (client, dir, NULL))
 		panel_profile_copy_defaults (client, dir);
 
+	panel_compatibility_migrate_panel_id_list (client);
+
 	key = panel_gconf_sprintf ("%s/general", dir);
 	gconf_client_add_dir (client, key, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
-	panel_compatibility_migrate_panel_id_list (client);
-
 	panel_profile_load_list (client,
 				 dir,
 				 PANEL_GCONF_TOPLEVELS,
-				 panel_profile_load_toplevel,
+				 panel_profile_load_and_show_toplevel,
 				 (GConfClientNotifyFunc) panel_profile_toplevel_id_list_notify);
 	panel_profile_load_list (client,
 				 dir,
@@ -2135,3 +2255,61 @@ panel_profile_set_show_program_list (gboolean show_program_list)
 	key = panel_gconf_general_key (current_profile, "show_program_list");
 	gconf_client_set_bool (client, key, show_program_list, NULL);
 }
+
+gboolean
+panel_profile_is_writable_show_program_list (void)
+{
+	GConfClient *client;
+	const char  *key;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_gconf_general_key (current_profile, "show_program_list");
+	return gconf_client_key_is_writable (client, key, NULL);
+}
+
+/* True if all the move keys are writable.  This is kind of
+   a blanket statement.  For example we may only lock one of them
+   down and that would prevent the user from moving the panel at
+   all, but really, making this truly fine grained is probably too
+   much crack */
+gboolean
+panel_profile_can_be_moved_freely (PanelToplevel *toplevel)
+{
+	const char *key;
+	GConfClient *client;
+	
+	if ( ! panel_profile_is_writable_toplevel_orientation (toplevel))
+		return FALSE;
+
+	client = panel_gconf_get_client ();
+
+	key = panel_profile_get_toplevel_key (toplevel, "screen");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	key = panel_profile_get_toplevel_key (toplevel, "monitor");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	/* For expanded panels we don't really have to check 
+	   x and y */
+	if (panel_toplevel_get_expand (toplevel))
+		return TRUE;
+
+	key = panel_profile_get_toplevel_key (toplevel, "x");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	key = panel_profile_get_toplevel_key (toplevel, "x_centered");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	key = panel_profile_get_toplevel_key (toplevel, "y");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	key = panel_profile_get_toplevel_key (toplevel, "y_centered");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+
+	return TRUE;
+}
